import matplotlib.pyplot as plt
import matplotlib.axes._axes as axes
import matplotlib.figure as figure
from mpl_toolkits.axisartist.parasite_axes import HostAxes, ParasiteAxes
import numpy as np

# 支持中文
plt.rcParams['font.sans-serif'] = ['SimHei']  # 用来正常显示中文标签
plt.rcParams['axes.unicode_minus'] = False  # 用来正常显示负号

"""
roc曲线
"""

# ------------ 准备数据 ------------ #
x_ls, y_ls = np.linspace(0, 10, 10), np.random.uniform(0, 1, size=10)

# ------------ 准备画布与图层 ------------ #
fig = plt.figure(figsize=(4, 4))  # type:figure.Figure

# 通过下面语句指定axes在fig中的位置和大小
# axes = fig.add_axes([left, bottom, width, height])
ax1 = fig.add_subplot()  # type: axes.Axes
# ax.twinx() 生成并返回关于y轴对称的一个axes
ax2 = ax1.twinx()  # type: axes.Axes

# ------------ 绘制图形元素 ------------ #
ax1.plot(x_ls, -y_ls, color='r')
ax2.plot(x_ls, y_ls, color='g')

plt.show()


def func(cfm, axis_paras, to_sort_x=False):
    """
    cfm dict 混淆矩阵
        axis_paras: dict {"x": {"key": key_0_in_cfm}  "y_0": {"key": key_1_in_cfm} }
    """
    x_paras = axis_paras.get("x", None)
    assert isinstance(x, (dict,))
    y_names = sorted([i for i in list(axis_paras.keys()) if i.startswith("y")],
                     key=lambda x: int(x.split("_", -1)[-1]) if "_" in x else -1)
    assert len(y_names) + 1 == len(axis_paras)
    y_paras_ls = [axis_paras[i] for i in y_names]

    # ------------ 准备画布与图层 ------------ #
    fig = plt.figure(figsize=(4, 4))  # type:figure.Figure

    # 添加 axes，并指定其在fig中的位置和大小
    axes_ls = []
    # axes = fig.add_axes([left, bottom, width, height])
    axes_ls.append(fig.add_axes([0.15, 0.1, 0.65, 0.8], axes_class=HostAxes))
    # for# https://www.runoob.com/python/python-func-sorted.html



import pytest
import numpy as np
from kevin.patches.for_test import check_consistency
from kevin.computer_science.algorithm.search import binary_search


def test_binary_search():
    print("test search.binary_search()")

    # check _binary_search
    for _ in range(1000):
        # 随机构建输入
        ls = np.random.rand(100).tolist()
        value = ls.pop(-1)
        # 查找
        index_0 = binary_search(ls=ls, value=value, is_sorted=False)
        # 标准答案
        ls.append(value)
        ls.sort()
        index_1 = ls.index(value)
        # 检查
        check_consistency(index_0, index_1)



import copy
import torch
import numpy as np


class Exponential_Moving_Average:
    def __init__(self, **kwargs):
        """
            滑动平均计数器
                支持为每个输入数据配置不同的权重
                参考： https://www.cnblogs.com/wuliytTaotao/p/9479958.html

            参数：
                keep_ratio:             <float> 对历史值的保留比例。
                                            其意义为： 大致等于计算过去 1/keep_ratio 个数据的平均值
                                            默认为 0.99
                                            当设置为 0 时相当于仅保留最新的数据
                bias_correction:        <boolean> 是否开启偏差修正。
                                            默认为 True
                指定输入数据的格式，有三种方式：
                    1. 显式指定数据的形状和所在设备等。
                        data_format:        <dict of paras>
                                其中需要包含以下参数：
                                    type_:              <str>
                                                            "numpy":        np.ndarray
                                                            "torch":        torch.tensor
                                    shape:              <list of integers>
                                    device:             <torch.device>
                                    dtype:              <torch.dtype>
                    2. 根据输入的数据，来推断出形状、设备等。
                        like:               <torch.tensor / np.ndarray>
                    3. 均不指定 data_format 和 like，此时将等到第一次调用 add_element()/add_sequence() 时再根据输入来自动推断。
                    以上三种方式，默认选用最后一种。
                    如果三种方式同时被指定，则优先级与对应方式在上面的排名相同。
        """

        # 默认参数
        paras = {
            # 超参数
            "keep_ratio": 0.99,
            "bias_correction": True,
            # 指定 tensor 的形状、设备
            "data_format": None,
            #
            "like": None,
        }

        # 获取参数
        paras.update(kwargs)

        # 校验参数
        assert isinstance(paras["keep_ratio"], (int, float,)) and 0 <= paras["keep_ratio"] <= 1
        #
        self.paras = paras
        self.value = self._init_value(like=paras["like"], data_format=paras["data_format"])
        self.state = dict(
            total_nums=0,
            bias_fix=1,
        )

    @staticmethod
    def _init_value(like=None, data_format=None):
        if like is not None:
            if torch.is_tensor(like):
                value = torch.zeros_like(like)
            elif isinstance(like, (np.ndarray,)):
                value = np.zeros_like(like)
            else:
                raise ValueError("paras 'like' should be np.ndarray or torch.tensor")
        elif data_format is not None:
            assert isinstance(data_format, (dict,)) and "type_" in data_format and "shape" in data_format
            k_s = copy.deepcopy(data_format)
            k_s.pop("type_")
            k_s.pop("shape")
            if data_format["type_"] == "torch":
                value = torch.zeros(size=data_format["shape"], **k_s)
            else:
                value = np.zeros(shape=data_format["shape"], **k_s)
        else:
            value = None

        return value

    def add_sequence(self, item_ls, weight_ls=None):
        if weight_ls is not None:
            if isinstance(weight_ls, (int, float,)):
                weight_ls = [weight_ls] * len(item_ls)
            assert len(weight_ls) == len(item_ls)
            for item, weight in enumerate(item_ls, weight_ls):
                self.add_element(item, weight)
        else:
            for item in item_ls:
                self.add_element(item)

    def add_element(self, item, weight=1):
        if self.value is None:
            self.value = self._init_value(like=item)
        new_ratio = (1 - self.paras["keep_ratio"]) * weight
        keep_ratio = (1 - new_ratio)
        self.value = keep_ratio * self.value + new_ratio * item
        #
        self.state["total_nums"] += 1
        self.state["bias_fix"] *= keep_ratio

    def get_value(self, bias_correction=None):
        assert self.value is not None, \
            f'Lack of initial value, pls invoke add_element()/add_sequence() first'
        bias_correction = self.paras["bias_correction"] if bias_correction is None else bias_correction
        if bias_correction:
            return self.value / (1 - self.state["bias_fix"] + 1e-10)
        else:
            return self.value

    def clear(self):
        if self.value is not None:
            self.value[:] = 0
        self.state = dict(
            total_nums=0,
            bias_fix=1,
        )


if __name__ == '__main__':
    seq = list(torch.tensor(range(1, 10)))
    ema = Exponential_Moving_Average(keep_ratio=0.9, bias_correction=True)
    for i, item in enumerate(seq):
        ema.add_element(item=item)
        print(i, item, ema.get_value(), ema.state["bias_fix"])



from setuptools import setup, find_packages

setup(
    name="kevin_toolbox",
    version="0.2.9",
    author="kevin hsu",
    author_email="xukaiming1996@163.com",
    description="一个常用的工具代码包集合",
    long_description=f'add norm() and svd() to patches.for_torch.compatible.linalg; \n'
                     f'add test cases for patches.for_torch; \n'
                     f'add normal_indices_generator() to math.dimension.coordinates; \n'
                     f'update math.dimension.coordinates.generate() to support pattern: normal; \n'
                     f'add Sequential to patches.for_torch.compatible.nn; \n'
                     f'add patches.for_matplotlib; \n'
                     f'add Exponential_Moving_Average to computer_science.algorithm.statistician; ',
    # 项目主页
    url="https://github.com/cantbeblank96/kevin_toolbox",
    #
    classifiers=[
        # 许可证信息
        'License :: OSI Approved :: MIT License',
        # 目标 Python 版本
        'Programming Language :: Python :: 3',
    ],

    #
    packages=find_packages(),

    # # 默认只添加 .py 文件，如果需要添加其他文件则需要令 include_package_data 为 True
    # include_package_data=True,
    # # 需要添加的额外文件列表，比如这里就表示将添加 images/raw 下的所有 .png 文件和 images/2k 下的所有 .jpg 文件
    # package_data={
    #     "images": ['raw/*.png', '2k/*.jpg'],
    # },
    # # 需要排除的文件
    # exclude_package_data={
    #     "images": ['raw/233.png'],
    # },

    # 依赖的python版本
    python_requires='>=3.6',

    # 表明当前模块依赖哪些包，若环境中没有，则会从pypi中下载安装
    install_requires=[
        'torch>=1.10.0',
        'numpy>=1.19.0',
        "matplotlib>=3.0"
    ],

    # 仅在测试时需要使用的依赖，在正常发布的代码中是没有用的。
    # 在执行python setup.py test时，可以自动安装这三个库，确保测试的正常运行。
    tests_require=[
        'pytest>=6.2.5',
        'line-profiler>=3.5',
    ],
)
